/**
 * Assignment #7: Disk Scheduling Algorithms
 * Sample Solution
 *
 * Instructor: Ron Mak
 * CMPE 142 Operating Systems
 * Computer Engineering Department
 * San Jose State University
 */
#ifndef SCHEDULINGALGORITHM_H_
#define SCHEDULINGALGORITHM_H_

#include <string>
#include <vector>
#include "Grapher.h"

using namespace std;

class SchedulingAlgorithm
{
public:
    SchedulingAlgorithm(const string name, const int max_cylinder)
        : max_cylinder(max_cylinder), name(name) {}

    virtual ~SchedulingAlgorithm() {}

    void schedule(const vector<int> requests, const int starting)
    {
        int name_length = name.length();

        cout << endl;
        for (int i = 0; i < name_length; i++) cout << "-";
        cout << endl;
        cout << name << endl;
        for (int i = 0; i < name_length; i++) cout << "-";
        cout << endl << endl;

        cout << "Requests:";
        for (int cylinder : requests) cout << setw(5) << cylinder;
        cout << endl;
        cout << "Starting:" << setw(5) << starting << endl;

        movement.push_back(starting);
        move(requests);

        cout << "Movement:";
        for (int i = 1; i < movement.size(); i++) cout << setw(5) << movement[i];
        cout << endl;

        grapher.graph(movement);

        cout << endl;
        cout << name << " total head movement: " << head_movement()
             << " cylinders" << endl;
    }

protected:
    int max_cylinder;
    vector<int> movement;

    virtual void move(vector<int> requests) = 0;

private:
    string name;
    Grapher grapher;

    int head_movement()
    {
        int total = 0;

        for (int i = 1; i < movement.size(); i++)
        {
            total += abs(movement[i] - movement[i-1]);
        }

        return total;
    }
};

#endif /* SCHEDULINGALGORITHM_H_ */
